/*
 * playlistsearchfilterlineedit.cpp
 *
 *  Created on: 14.06.2010
 *      Author: darkstar
 */

#include <qtimer.h>
#include <qapplication.h>

#include "playlistsearchfilterlineedit.h"
#include "mediadatabase.h"
#include "playlistaspect.h"
#include "playlistaspectsearchfilter.h"
#include "configuration.h"

#include "debug.h"

PlayListSearchFilterLineEdit::PlayListSearchFilterLineEdit(QWidget *parent)
    : QLineEdit(parent),
      aspectSource_(NULL),
      newFilterText_("")
{
#ifdef QT4
    setInputMethodHints(Qt::ImhNoPredictiveText);
#endif

    filterEditTimer_ = new QTimer(this);
    connect(filterEditTimer_, SIGNAL(timeout()), this, SLOT(filterEditBoxTimedUpdate()));
    connect(this, SIGNAL(textChanged(const QString&)), this, SLOT(filterEditBoxTextChanged(const QString&)));
}

PlayListSearchFilterLineEdit::~PlayListSearchFilterLineEdit()
{

}

void PlayListSearchFilterLineEdit::filterEditBoxTextChanged(const QString &text)
{
    if (!aspectSource_) return;

    newFilterText_ = text;

    if (filterEditTimer_->isActive())
        filterEditTimer_->stop();

#ifndef QT4
    filterEditTimer_->start(qConfig.playlistAutoTriggerFilterInterval, true);
#else
    filterEditTimer_->setSingleShot(true);
    filterEditTimer_->start(qConfig.playlistAutoTriggerFilterInterval);
#endif
}

void PlayListSearchFilterLineEdit::filterEditBoxTimedUpdate()
{
    if (!aspectSource_) return;

    aspectSource_->setFilterText(newFilterText_);
}

void PlayListSearchFilterLineEdit::attachTo(PlayListAspectSearchFilter *source)
{
    DGUARDMETHODTIMED;
    detach();

    if (source)
    {
        blockSignals(true);
        setText(source->filterText());
        blockSignals(false);
        aspectSource_ = source;

        connect(aspectSource_, SIGNAL(updated(const QString&)), this, SLOT(indirectSetText(const QString&)));
    }
}

void PlayListSearchFilterLineEdit::detach()
{
    DGUARDMETHODTIMED;
    if (aspectSource_)
    {
        disconnect(aspectSource_, SIGNAL(updated(const QString&)), this, SLOT(indirectSetText(const QString&)));

        aspectSource_ = NULL;

        newFilterText_ = "";
        blockSignals(true);
        setText("");
        blockSignals(false);
    }
}

void PlayListSearchFilterLineEdit::indirectSetText(const QString& text)
{
    blockSignals(true);
    setText(text);
    blockSignals(false);
}
